<?php

namespace HS\Providers;

use App\Actions\Fortify\CreateNewUser;
use App\Actions\Fortify\ResetUserPassword;
use App\Actions\Fortify\UpdateUserPassword;
use App\Actions\Fortify\UpdateUserProfileInformation;
use Illuminate\Cache\RateLimiting\Limit;
use Illuminate\Contracts\View\View;
use Illuminate\Contracts\View\Factory;
use Illuminate\Http\RedirectResponse;
use Illuminate\Contracts\Container\BindingResolutionException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\RateLimiter;
use Illuminate\Support\ServiceProvider;
use Laravel\Fortify\Fortify;
use Illuminate\Support\Facades\Cookie;
use InvalidArgumentException;
use Laravel\Fortify\Contracts\LogoutResponse;
use Symfony\Component\Routing\Exception\RouteNotFoundException;

class FortifyServiceProvider extends ServiceProvider
{
    /**
     * Register any application services.
     *
     * @return void
     */
    public function register()
    {
        // Set a custom redirect when logging out.
        $this->app->instance(LogoutResponse::class, new class implements LogoutResponse {
            public function toResponse($request)
            {
                if (hs_setting('cAUTHTYPE', 'internal') == 'saml') {
                    return redirect()->route('altlogin');
                }
                return redirect('/login');
            }
        });
    }

    /**
     * Bootstrap any application services.
     *
     * @return void
     */
    public function boot()
    {
        // Fortify::createUsersUsing(CreateNewUser::class);
        // Fortify::updateUserProfileInformationUsing(UpdateUserProfileInformation::class);
        // Fortify::updateUserPasswordsUsing(UpdateUserPassword::class);
        // Fortify::resetUserPasswordsUsing(ResetUserPassword::class);

        Fortify::loginView(function () {
            return $this->loginView();
        });

        Fortify::confirmPasswordView(function () {
            return view('auth.passwords.confirm');
        });

        Fortify::twoFactorChallengeView(function () {
            return view('auth.passwords.two-factor-challenge');
        });

        RateLimiter::for('login', function (Request $request) {
            $email = (string) $request->email;
            return Limit::perMinute(5)->by($email.$request->ip());
        });

        RateLimiter::for('two-factor', function (Request $request) {
            return Limit::perMinute(5)->by($request->session()->get('login.id'));
        });
    }

    /**
     * Build our custom login view
     */
    protected function loginView()
    {
        if (request()->input('mobileauth')) {
            Cookie::queue('mobile-transact', 'true', 60);
        }

        if (hs_setting('cAUTHTYPE', 'internal') == 'saml') {
            // Prevent redirect loops:
            // If user is logged into portal, redirect to the primary portal
            // If they're already auth'ed as an admin, they'd be redirected already
            if (auth('portal')->check()) {
                return redirect()->to(cHOST.'/index.php?pg=request.history');
            } else {
                // Else assume they want to login as an admin
                return redirect()->route('saml2_login', 'hs');
            }
        }

        return view('auth.login', array_merge(
            $this->getLatestNews(),
            ['mobileauth' => false],
        ));
    }

    /**
     * Get the latest HelpSpot News
     * @return array
     * @throws BindingResolutionException
     */
    protected function getLatestNews()
    {
        $article = getHelpSpotRssArticleLink();

        if (! $article) {
            return ['article_link' => '', 'article_title' => ''];
        }

        return [
            'article_link'=> $article['link'].'?m='.createBlogLinkMetaQueryString(),
            'article_title' => $article['title'],
        ];
    }
}
